local name = "CVEdit"
local version = "0.1"
local debugOutput = false

local options = {
  { "CV", VALUE, 1, 1, 32 },
  { "Point", VALUE, 1, 1, 17},
	{ "Source", SOURCE, 1 },
	{ "Enabled", BOOL, 0  },
  { "Color", COLOR, ORANGE },
}

-- create widget
local function create(zone, options)
  local widget = {
    zone = zone,
    options = options
  }
	
	initWidget(widget)

  return widget
end

-- process widget settings update
local function update(widget, options)
  widget.options = options
	
	initWidget(widget)
end

-- init widget's status variables
function initWidget(widget)
	widget.cvIdx = widget.options.CV - 1
	
	if(widget.cvIdx) ~= nil then
	  widget.cvBase = model.getCurve(widget.cvIdx).y[0] and 0 or 1
	end
	
	widget.sourceName = getFieldInfo(widget.options.Source).desc
	widget.value = getInputValue(widget)
	widget.lastValue = widget.value
	widget.error = ""
	
	debugCurve(widget)
end

-- no background processing
local function background(widget)
end

-- main processing loop
local function refresh(widget, event, touchState)
	setCurvePoint(widget)
	updateDisplay(widget)
end

-- update requested point
function setCurvePoint(widget)
	local cv = model.getCurve(widget.cvIdx)

  if(cv == nil) then
		widget.error = "CV not found"
		return
	end
	
	if(widget.options.Point > cv.points) then
		widget.error = "point > nPoints"
		return
	end

	widget.value = getInputValue(widget)
	
	if(widget.value == widget.lastValue) then
		return
	end
	
	widget.lastValue = widget.value

	cv.y[widget.cvBase + widget.options.Point - 1] = widget.value
	
	model.setCurve(widget.cvIdx, cv)

	debugCurve(widget)
end

-- update widget's display
function updateDisplay(widget)
	lcd.drawText(0, 0, name .. " " .. version .. " - " .. widget.sourceName, widget.options.Color)

	if widget.options.Enabled == 0 then
		widget.error = "Widget disabled"
	end
	
	if widget.error == "" then
		lcd.drawText(0, 20, "CV".. widget.cvIdx + 1 .." P" .. widget.options.Point .. "   " .. widget.value, widget.options.Color)
	else
		lcd.drawText(0, 20, widget.error, widget.options.Color)
	end
end

-- debug print curve's y-values
function debugCurve(widget)
	if(debugOutput == false) then
		return
	end
	
	print("*******")

  local cv = model.getCurve(widget.cvIdx)

  if(cv ~= nil) then 
	  for point = 1, cv.points do
	    print(cv.y[widget.cvBase + point - 1])
    end
	end
end

-- scale source value to +-100
function getInputValue(widget)
	return math.floor((getValue(widget.options.Source) * 100) / 1024)
end

return {
  name = name,
  options = options,
  create = create,
  update = update,
  refresh = refresh,
  background = background
}